"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectDefinition = void 0;
const decorator_1 = require("@midwayjs/decorator");
const properties_1 = require("./properties");
const objectCreator_1 = require("./objectCreator");
/* tslint:disable:variable-name */
class ObjectDefinition {
    constructor() {
        this._attrs = new Map();
        this._asynchronous = false;
        // 对象定义默认需要自动装配
        this._autowire = true;
        this.scope = decorator_1.ScopeEnum.Singleton;
        this.creator = null;
        this.id = null;
        this.name = null;
        this.initMethod = null;
        this.destroyMethod = null;
        this.constructMethod = null;
        this.constructorArgs = [];
        this.path = null;
        this.export = null;
        this.dependsOn = [];
        this.properties = new properties_1.ObjectProperties();
        this.namespace = '';
        this.handlerProps = [];
        this.creator = new objectCreator_1.ObjectCreator(this);
    }
    set autowire(autowire) {
        this._autowire = autowire;
    }
    isAutowire() {
        return this._autowire;
    }
    set asynchronous(asynchronous) {
        this._asynchronous = asynchronous;
    }
    isAsync() {
        return this._asynchronous;
    }
    isSingletonScope() {
        return this.scope === decorator_1.ScopeEnum.Singleton;
    }
    isRequestScope() {
        return this.scope === decorator_1.ScopeEnum.Request;
    }
    hasDependsOn() {
        return this.dependsOn.length > 0;
    }
    hasConstructorArgs() {
        return this.constructorArgs.length > 0;
    }
    getAttr(key) {
        return this._attrs.get(key);
    }
    hasAttr(key) {
        return this._attrs.has(key);
    }
    setAttr(key, value) {
        this._attrs.set(key, value);
    }
}
exports.ObjectDefinition = ObjectDefinition;
/* tslint:enable:variable-name */
//# sourceMappingURL=objectDefinition.js.map