"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractExpressLikeValue = exports.extractKoaLikeValue = void 0;
const decorator_1 = require("@midwayjs/decorator");
const extractKoaLikeValue = (key, data) => {
    if (decorator_1.ALL === data) {
        data = undefined;
    }
    return async function (ctx, next) {
        switch (key) {
            case decorator_1.RouteParamTypes.NEXT:
                return next;
            case decorator_1.RouteParamTypes.BODY:
                return data && ctx.request.body
                    ? ctx.request.body[data]
                    : ctx.request.body;
            case decorator_1.RouteParamTypes.PARAM:
                return data ? ctx.params[data] : ctx.params;
            case decorator_1.RouteParamTypes.QUERY:
                return data ? ctx.query[data] : ctx.query;
            case decorator_1.RouteParamTypes.HEADERS:
                return data ? ctx.get(data) : ctx.headers;
            case decorator_1.RouteParamTypes.SESSION:
                return data ? ctx.session[data] : ctx.session;
            case decorator_1.RouteParamTypes.FILESTREAM:
                return ctx.getFileStream && ctx.getFileStream(data);
            case decorator_1.RouteParamTypes.FILESSTREAM:
                return ctx.multipart && ctx.multipart(data);
            case decorator_1.RouteParamTypes.REQUEST_PATH:
                return ctx['path'];
            case decorator_1.RouteParamTypes.REQUEST_IP:
                return ctx['ip'];
            case decorator_1.RouteParamTypes.QUERIES:
                if (ctx.queries) {
                    return data ? ctx.queries[data] : ctx.queries;
                }
                else {
                    return data ? ctx.query[data] : ctx.query;
                }
            default:
                return null;
        }
    };
};
exports.extractKoaLikeValue = extractKoaLikeValue;
const extractExpressLikeValue = (key, data) => {
    if (decorator_1.ALL === data) {
        data = undefined;
    }
    return async function (req, res, next) {
        switch (key) {
            case decorator_1.RouteParamTypes.NEXT:
                return next;
            case decorator_1.RouteParamTypes.BODY:
                return data && req.body ? req.body[data] : req.body;
            case decorator_1.RouteParamTypes.PARAM:
                return data ? req.params[data] : req.params;
            case decorator_1.RouteParamTypes.QUERY:
                return data ? req.query[data] : req.query;
            case decorator_1.RouteParamTypes.HEADERS:
                return data ? req.get(data) : req.headers;
            case decorator_1.RouteParamTypes.SESSION:
                return data ? req.session[data] : req.session;
            case decorator_1.RouteParamTypes.FILESTREAM:
                return req.getFileStream && req.getFileStream(data);
            case decorator_1.RouteParamTypes.FILESSTREAM:
                return req.multipart && req.multipart(data);
            case decorator_1.RouteParamTypes.REQUEST_PATH:
                return req['baseUrl'];
            case decorator_1.RouteParamTypes.REQUEST_IP:
                return req['ip'];
            case decorator_1.RouteParamTypes.QUERIES:
                if (req.queries) {
                    return data ? req.queries[data] : req.queries;
                }
                else {
                    return data ? req.query[data] : req.query;
                }
            default:
                return null;
        }
    };
};
exports.extractExpressLikeValue = extractExpressLikeValue;
//# sourceMappingURL=webRouterParam.js.map